import 'package:eh_redux/screens/check_update/store.dart';
import 'package:eh_redux/utils/launch.dart';
import 'package:eh_redux/widgets/center_progress_indicator.dart';
import 'package:filesize/filesize.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_mobx/flutter_mobx.dart';

enum _CheckUpdateAction {
  openInBrowser,
}

class CheckUpdateScreen extends StatefulWidget {
  const CheckUpdateScreen({Key key}) : super(key: key);

  static const routeName = '/checkUpdate';

  @override
  _CheckUpdateScreenState createState() => _CheckUpdateScreenState();
}

class _CheckUpdateScreenState extends State<CheckUpdateScreen> {
  CheckUpdateStore _store;

  @override
  void initState() {
    super.initState();
    _store = CheckUpdateStore()..check();
  }

  @override
  Widget build(BuildContext context) {
    return Observer(
      builder: (context) {
        final actions = <Widget>[];

        if (_store.releaseFuture.value != null) {
          actions.add(PopupMenuButton<_CheckUpdateAction>(
            onSelected: (action) {
              switch (action) {
                case _CheckUpdateAction.openInBrowser:
                  tryLaunch(_store.releaseFuture.value.htmlUrl);
                  break;
              }
            },
            itemBuilder: (context) => const [
              PopupMenuItem(
                value: _CheckUpdateAction.openInBrowser,
                child: Text('Open in browser'),
              ),
            ],
          ));
        }

        return Scaffold(
          appBar: AppBar(
            title: const Text('Check Updates'),
            actions: actions,
          ),
          body: _buildBody(),
        );
      },
    );
  }

  Widget _buildCenterText(String text) {
    final theme = Theme.of(context);

    return Center(
      child: Text(text, style: theme.textTheme.headline6),
    );
  }

  Widget _buildBody() {
    final theme = Theme.of(context);

    return SafeArea(
      top: false,
      child: Observer(
        builder: (context) {
          if (_store.status == UpdateStatus.pending) {
            return const CenterProgressIndicator();
          }

          if (_store.status == UpdateStatus.failed) {
            return _buildCenterText('Failed to check updates.');
          }

          final release = _store.releaseFuture.value;

          return ListView(
            padding: const EdgeInsets.all(16),
            children: <Widget>[
              Row(
                children: <Widget>[
                  Expanded(
                    child: Text(
                      release.name,
                      style: theme.textTheme.headline6,
                    ),
                  ),
                  _buildDownloadButton(release),
                ],
              ),
              Html(data: release.body),
            ],
          );
        },
      ),
    );
  }

  Widget _buildDownloadButton(GitHubRelease release) {
    final asset = release.assets.firstWhere((asset) =>
        asset.contentType == 'application/vnd.android.package-archive' &&
        asset.state == 'uploaded' &&
        asset.name.contains('arm64-v8a'));

    if (asset == null || _store.status != UpdateStatus.canUpdate) {
      return const FlatButton(
        onPressed: null,
        child: Text('Up to date'),
      );
    }

    return RaisedButton.icon(
      onPressed: () {
        tryLaunch(asset.browserDownloadUrl);
      },
      icon: Icon(Icons.file_download),
      label: Text('Download (${filesize(asset.size)})'),
    );
  }
}
